@@ -3,12 +3,12 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Callable
-from datetime import UTC, datetime
+from collections.abc import AsyncIterator, Callable
+from contextlib import asynccontextmanager
 from json.decoder import JSONDecodeError
-from typing import Final
+from typing import Any, Final
 
-from httpx import AsyncClient, Cookies, HTTPStatusError, Request, Response
+from httpx import AsyncClient, Cookies, HTTPStatusError, Response
 from mashumaro.codecs.json import json_decode
 from mashumaro.exceptions import (
     MissingField,
@@ -51,17 +51,52 @@ class Client:
         self._httpx_client = httpx_client
 
         # Token information:
-        self._access_token: str | None = None
-        self._access_token_expires_at: datetime | None = None
         self._refresh_lock = asyncio.Lock()
-        self._refresh_token: str | None = None
-        self._refresh_token_callbacks: list[Callable[[str], None]] = []
         self._refreshing = False
+        self._session_id: str | None = None
+        self._session_id_callbacks: list[Callable[[str], None]] = []
 
         # Define endpoints:
         self.llm = LLMEndpoint(self._request_and_validate)
-        self.prompt = PromptEndpoint(self._request_and_validate)
-        self.thread = ThreadEndpoint(self._request, self._request_and_validate)
+        self.prompt = PromptEndpoint(self._request_and_validate, self._stream)
+        self.thread = ThreadEndpoint(self._request_and_validate)
+
+    def _create_cookie_jar(self, raw_cookies: dict[str, Any] | None) -> Cookies:
+        """Create a cookie jar from raw cookies.
+
+        Args:
+        ----
+            raw_cookies: The raw cookies to use.
+
+        Returns:
+        -------
+            A cookie jar.
+
+        """
+        if not raw_cookies:
+            raw_cookies = {}
+        if self._session_id:
+            raw_cookies["session"] = self._session_id
+        return Cookies(raw_cookies)
+
+    @asynccontextmanager
+    async def _get_httpx_client(self) -> AsyncIterator[AsyncClient]:
+        """Get an HTTPX client.
+
+        Returns
+        -------
+            An HTTPX client.
+
+        """
+        if running_client := self._httpx_client and not self._httpx_client.is_closed:
+            client = self._httpx_client
+        else:
+            client = AsyncClient()
+
+        yield client
+
+        if not running_client:
+            await client.aclose()
 
     async def _request(
         self,
@@ -69,11 +104,11 @@ class Client:
         endpoint: str,
         *,
         headers: dict[str, str] | None = None,
-        cookies: Cookies | None = None,
+        cookies: dict[str, str] | None = None,
         params: dict[str, str] | None = None,
-        json: dict[str, str] | None = None,
+        json: dict[str, Any] | None = None,
     ) -> Response:
-        """Make a request to the Liminal API server and return a Response.
+        """Make a request to the Liminal API server and return a response.
 
         Args:
         ----
@@ -93,44 +128,32 @@ class Client:
             RequestError: If the response fails for any reason.
 
         """
-        utcnow = datetime.now(tz=UTC)
-        if self._access_token_expires_at and utcnow >= self._access_token_expires_at:
-            LOGGER.debug("Access token expired, refreshing...")
-            self._access_token = None
-            self._access_token_expires_at = None
-            await self.authenticate_from_refresh_token()
-
         url = f"{self._api_server_url}{endpoint}"
+        cookie_jar = self._create_cookie_jar(cookies)
 
-        if not headers:
-            headers = {}
-        if self._access_token:
-            headers["Authorization"] = f"Bearer {self._access_token}"
-
-        if running_client := self._httpx_client and not self._httpx_client.is_closed:
-            client = self._httpx_client
-        else:
-            client = AsyncClient()
-
-        request = Request(
-            method, url, headers=headers, cookies=cookies, params=params, json=json
-        )
-        response = await client.send(request)
-
-        try:
-            response.raise_for_status()
-        except HTTPStatusError as err:
-            msg = (
-                f"Error while sending request to {url}: {err.response.content.decode()}"
+        async with self._get_httpx_client() as client:
+            response = await client.request(
+                method,
+                url,
+                headers=headers,
+                cookies=cookie_jar,
+                params=params,
+                json=json,
+                timeout=DEFAULT_REQUEST_TIMEOUT,
             )
-            raise RequestError(msg) from err
 
-        if not running_client:
-            await client.aclose()
+            try:
+                response.raise_for_status()
+            except HTTPStatusError as err:
+                msg = (
+                    f"Error while sending request to {url}: "
+                    f"{err.response.content.decode()}"
+                )
+                raise RequestError(msg) from err
 
-        LOGGER.debug("Received data from %s: %s", url, response.content)
+            LOGGER.debug("Received data from %s: %s", url, response.content)
 
-        return response
+            return response
 
     async def _request_and_validate(
         self,
@@ -139,9 +162,9 @@ class Client:
         expected_response_type: type[ValidatedResponseT],
         *,
         headers: dict[str, str] | None = None,
-        cookies: Cookies | None = None,
+        cookies: dict[str, str] | None = None,
         params: dict[str, str] | None = None,
-        json: dict[str, str] | None = None,
+        json: dict[str, Any] | None = None,
     ) -> ValidatedResponseT:
         """Make a request to the Liminal API server and validate the response.
 
@@ -179,30 +202,69 @@ class Client:
             msg = f"Could not validate response: {err}"
             raise RequestError(msg) from err
 
-    def _save_tokens_from_auth_response(self, auth_response: Response) -> None:
-        """Save tokens from an auth response.
+    def _save_session_id_from_auth_response(self, auth_response: Response) -> None:
+        """Save a session cookie value from an auth response.
 
         Args:
         ----
             auth_response: The response from an auth request.
 
         """
-        LOGGER.debug("Saving tokens from auth response")
-        self._access_token = auth_response.cookies["accessToken"]
-        self._access_token_expires_at = datetime.fromtimestamp(
-            int(auth_response.cookies["accessTokenExpiresAt"]) / 1000, tz=UTC
-        )
-        self._refresh_token = auth_response.cookies["refreshToken"]
+        LOGGER.debug("Saving session cookie from auth response")
+        self._session_id = auth_response.cookies["session"]
 
-        for callback in self._refresh_token_callbacks:
-            callback(self._refresh_token)
+        if self._session_id:
+            for callback in self._session_id_callbacks:
+                callback(self._session_id)
 
-    def add_refresh_token_callback(
+    async def _stream(
+        self,
+        method: str,
+        endpoint: str,
+        *,
+        headers: dict[str, str] | None = None,
+        cookies: dict[str, str] | None = None,
+        params: dict[str, str] | None = None,
+        json: dict[str, Any] | None = None,
+    ) -> AsyncIterator[str]:
+        """Make a request to the Liminal API server and return a streaming response.
+
+        Args:
+        ----
+            method: The HTTP method to use.
+            endpoint: The endpoint to request.
+            headers: The headers to use.
+            cookies: The cookies to use.
+            params: The query parameters to use.
+            json: The JSON body to use.
+
+        Returns:
+        -------
+            An AsyncIterator containing the raw JSON response chunks.
+
+        """
+        url = f"{self._api_server_url}{endpoint}"
+        cookie_jar = self._create_cookie_jar(cookies)
+
+        # pylint: disable=contextmanager-generator-missing-cleanup
+        async with self._get_httpx_client() as client, client.stream(
+            method,
+            url,
+            headers=headers,
+            cookies=cookie_jar,
+            params=params,
+            json=json,
+        ) as resp:
+            async for line in resp.aiter_lines():
+                LOGGER.info("Received line of streaming response: %s", line)
+                yield line
+
+    def add_session_id_callback(
         self, callback: Callable[[str], None]
     ) -> Callable[[], None]:
-        """Add a callback to be called when a new refresh token is generated.
+        """Add a callback to be called when a new session is generated.
 
-        The purpose of this is to allow the user to save the refresh token to a
+        The purpose of this is to allow the user to save the session ID to a
         persistent store using their own callback method.
 
         Args:
@@ -214,11 +276,11 @@ class Client:
             A method to cancel and remove the callback.
 
         """
-        self._refresh_token_callbacks.append(callback)
+        self._session_id_callbacks.append(callback)
 
         def cancel() -> None:
             """Cancel and remove the callback."""
-            self._refresh_token_callbacks.remove(callback)
+            self._session_id_callbacks.remove(callback)
 
         return cancel
 
@@ -230,40 +292,40 @@ class Client:
             "/api/v1/auth/login/oauth/access-token",
             headers={"Authorization": f"Bearer {provider_access_token}"},
         )
-        self._save_tokens_from_auth_response(liminal_auth_response)
+        self._save_session_id_from_auth_response(liminal_auth_response)
 
-    async def authenticate_from_refresh_token(
-        self, *, refresh_token: str | None = None
+    async def authenticate_from_session_id(
+        self, *, session_id: str | None = None
     ) -> None:
-        """Authenticate with the Liminal API server (using a refresh token).
+        """Authenticate with the Liminal API server (using a session).
 
         Args:
         ----
-            refresh_token: The refresh token to use. If not provided, the refresh token
-                that was used to authenticate the user initially will be used.
+            session_id: The session ID to use. If not provided, the session that was
+                used to authenticate the user initially will be used.
 
         Raises:
         ------
-            AuthError: If no refresh token is provided and the user has not been
-                authenticated yet.
+            AuthError: If no session is provided and the user has not been authenticated
+                yet.
 
         """
-        if not refresh_token:
-            refresh_token = self._refresh_token
+        if not session_id:
+            session_id = self._session_id
 
-        if not refresh_token:
-            msg = "No valid refresh token provided"
+        if not session_id:
+            msg = "No valid session ID provided"
             raise AuthError(msg)
 
         async with self._refresh_lock:
             self._refreshing = True
 
             try:
-                refresh_token_response = await self._request(
-                    "POST",
-                    "/api/v1/auth/refresh-token",
-                    cookies=Cookies({"refreshToken": refresh_token}),
+                session_id_response = await self._request(
+                    "GET",
+                    "/api/v1/users/me",
+                    cookies={"session": session_id},
                 )
-                self._save_tokens_from_auth_response(refresh_token_response)
+                self._save_session_id_from_auth_response(session_id_response)
             finally:
                 self._refreshing = False
